#ifndef PDF_LZW_H
#define PDF_LZW_H

#include <hammer/hammer.h>


typedef struct LZW_context_S
{
	/*
	 * Table for storing sequences represented by an LZW code
	 * 0-255, and 256 are special, representing literals, and the reset code. We could explicitly pre-fill them, but it's probably not necessary.
	 */
	HBytes * lzw_code_table[4096];

	/*
	 * Holds the next expected LZW code. We also use this for telling LZW_9bitcodeword, LZW_10bitcodeword, etc. apart. Parses fail if "next" is larger than what can be represented on that many bits.
	 */
	int next;

	/*
	 * Previous LZW code, used to construct the next string added to the table.
	 */
	uint64_t old;

	/*
	 * EarlyChange = 1 means the bit size is increased "one code early" (Early change = 0 is "code length increases shall be postponed as long as possible"
	 */
	int earlychange;
} LZW_context_T;

void init_LZW_parser();
HParseResult * parse_LZW_data(const uint8_t* input, size_t length);
void init_LZW_context(int earlychange);

#endif // PDF_LZW_H
